import numpy as np
from pyNastran.op2.op2 import OP2
from pyNastran.bdf.bdf import BDF
from pyNastran.bdf.mesh_utils.cut_model_by_plane import get_element_centroids, get_stations
import matplotlib.pyplot as plt

def plot_smt(x, force_sum, moment_sum, show=True):
    """plots the shear, moment, torque plots"""

    plt.close()
    #f, ax = plt.subplots()
    # ax = fig.subplots()
    fig = plt.figure(1)
    ax = fig.gca()
    ax.plot(x, force_sum[:, 0], '-*')
    ax.set_title('X vs. Axial')
    ax.set_xlabel('X')
    ax.set_ylabel('Axial')
    ax.grid(True)

    fig = plt.figure(2)
    ax = fig.gca()
    ax.plot(x, force_sum[:, 1], '-*')
    ax.set_title('X vs. Shear Y')
    ax.set_xlabel('X')
    ax.set_ylabel('Shear Y')
    ax.grid(True)

    fig = plt.figure(3)
    ax = fig.gca()
    ax.plot(x, force_sum[:, 2], '-*')
    ax.set_title('X vs. Shear Z')
    ax.set_xlabel('X')
    ax.set_ylabel('Shear Z')
    ax.grid(True)

    fig = plt.figure(4)
    ax = fig.gca()
    ax.plot(x, moment_sum[:, 0], '-*')
    ax.set_title('X vs. Torque')
    ax.set_xlabel('X')
    ax.set_ylabel('Torque')
    ax.grid(True)

    fig = plt.figure(5)
    ax = fig.gca()
    ax.plot(x, moment_sum[:, 1], '-*')
    ax.set_title('X vs. Moment Y')
    ax.set_xlabel('X')
    ax.set_ylabel('Moment Y')
    ax.grid(True)

    fig = plt.figure(6)
    ax = fig.gca()
    ax.plot(x, moment_sum[:, 2], '-*')
    ax.set_title('X vs. Moment Z')
    ax.set_xlabel('X')
    ax.set_ylabel('Moment Z')
    ax.grid(True)

    plt.show()




bdf_fname='simple_beam.bdf'
op2_fname='simple_beam.op2'

model=BDF()
model._stop_on_duplicate_error=False
model.set_cards(['CORD2R','GRID','CQUAD4','CTRIA3','MAT1','MAT8','PSHELL','PCOMP','PCOMPG'])
model.read_bdf(bdf_fname,punch=True)
op2=OP2()
op2.read_op2(op2_fname,skip_undefined_matrices=True)
e=3

gpforce = op2.grid_point_forces[10070]
out = model.get_xyz_in_coord_array(cid=0)
nid_cp_cd, xyz_cid0, xyz_cp, icd_transform, icp_transform = out
nids = nid_cp_cd[:, 0]
nid_cd = nid_cp_cd[:, [0, 2]]
eids, element_centroids_cid0 = get_element_centroids(model)
coord_out = model.coords[0]

#cid_p1 = 0 # start
#cid_p3 = 0 # end
#cid_p2 = 0 # coord
#p1-p2 defines the x-axis
#k is defined by the z-axis
#p1 = np.array([1354., 0., 0.]) # origin
#p2 = np.array([1354., 1245., 0.]) # xaxis
#p3 = np.array([1354., 1245., 0.]) # end
#zaxis = np.array([0., 0., 1.])
#method = 'Z-Axis Projection'
#idir = 0

#p1 = np.array([1354., 0., 0.]) # origin
#p2 = np.array([1354., 0., 1.]) # xzplane
#p3 = np.array([1354., 1245., 0.]) # end
#zaxis = np.array([0., 0., 1.])
#method = 'CORD2R'
#idir = 1 # x-direction in this rotated system


# axial
p1 = np.array([0., 0., 0.]) # origin
p2 = np.array([1., 0., 0.]) # xaxis
p3 = np.array([-500., 0., 0.]) # end
zaxis = np.array([0., 0., 1.])
method = 'Z-Axis Projection'
idir = 0

xyz1, xyz2, xyz3, i, k, coord_out, stations = get_stations(
    model, p1, p2, p3, zaxis,
    method=method, cid_p1=0, cid_p2=0, cid_p3=0,
    cid_zaxis=0, idir=idir, nplanes=40)

#stations=sumation_pts[:,0]

force_sum, moment_sum =gpforce.shear_moment_diagram(
    xyz_cid0, eids, nids, icd_transform,
    element_centroids_cid0,
    model.coords, nid_cd, stations, coord_out,
    idir=idir, itime=0, debug=True, log=model.log)#,sum_points=sumation_pts)
print(stations)

# i/j/k vector is nan
print(f'origin: {coord_out.origin}')
print(f'zaxis: {coord_out.e2}')
print(f'xzplane: {coord_out.e3}')
print(force_sum)
print(moment_sum)
plot_smt(stations, force_sum, moment_sum, show=False)